@@ -5,12 +5,13 @@
     SPDX-FileCopyrightText: 2000 David Faure <faure@kde.org>
     SPDX-FileCopyrightText: 2006 Kevin Ottens <ervin@kde.org>
 
-    SPDX-License-Identifier: LGPL-2.0-only
+    SPDX-License-Identifier: LGPL-2.0-or-later
 */
 
 #include "kjob.h"
 #include "kjob_p.h"
 
+#include "kcoreaddons_debug.h"
 #include "kjobuidelegate.h"
 
 #include <QEventLoop>
@@ -52,13 +53,16 @@ KJob::~KJob()
 void KJob::setUiDelegate(KJobUiDelegate *delegate)
 {
     Q_D(KJob);
-    if (delegate == nullptr || delegate->setJob(this)) {
+    if (!delegate) {
+        delete d->uiDelegate;
+        d->uiDelegate = nullptr;
+        return;
+    }
+
+    if (delegate->setJob(this)) {
         delete d->uiDelegate;
         d->uiDelegate = delegate;
-
-        if (d->uiDelegate) {
-            d->uiDelegate->connectJob(this);
-        }
+        d->uiDelegate->connectJob(this);
     }
 }
 
@@ -215,12 +219,22 @@ QString KJob::errorString() const
 
 qulonglong KJob::processedAmount(Unit unit) const
 {
-    return d_func()->processedAmount[unit];
+    if (unit >= UnitsCount) {
+        qCWarning(KCOREADDONS_DEBUG) << "KJob::processedAmount() was called on an invalid Unit" << unit;
+        return 0;
+    }
+
+    return d_func()->m_jobAmounts[unit].processedAmount;
 }
 
 qulonglong KJob::totalAmount(Unit unit) const
 {
-    return d_func()->totalAmount[unit];
+    if (unit >= UnitsCount) {
+        qCWarning(KCOREADDONS_DEBUG) << "KJob::totalAmount() was called on an invalid Unit" << unit;
+        return 0;
+    }
+
+    return d_func()->m_jobAmounts[unit].totalAmount;
 }
 
 unsigned long KJob::percent() const
@@ -247,10 +261,18 @@ void KJob::setErrorText(const QString &errorText)
 
 void KJob::setProcessedAmount(Unit unit, qulonglong amount)
 {
-    Q_D(KJob);
-    bool should_emit = (d->processedAmount[unit] != amount);
+    if (unit >= UnitsCount) {
+        qCWarning(KCOREADDONS_DEBUG) << "KJob::setProcessedAmount() was called on an invalid Unit" << unit;
+        return;
+    }
 
-    d->processedAmount[unit] = amount;
+    Q_D(KJob);
+
+    auto &[processed, total] = d->m_jobAmounts[unit];
+
+    const bool should_emit = (processed != amount);
+
+    processed = amount;
 
     if (should_emit) {
 #if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 80)
@@ -259,17 +281,25 @@ void KJob::setProcessedAmount(Unit unit, qulonglong amount)
         Q_EMIT processedAmountChanged(this, unit, amount, QPrivateSignal{});
         if (unit == d->progressUnit) {
             Q_EMIT processedSize(this, amount);
-            emitPercent(d->processedAmount[unit], d->totalAmount[unit]);
+            emitPercent(processed, total);
         }
     }
 }
 
 void KJob::setTotalAmount(Unit unit, qulonglong amount)
 {
-    Q_D(KJob);
-    bool should_emit = (d->totalAmount[unit] != amount);
+    if (unit >= UnitsCount) {
+        qCWarning(KCOREADDONS_DEBUG) << "KJob::setTotalAmount() was called on an invalid Unit" << unit;
+        return;
+    }
 
-    d->totalAmount[unit] = amount;
+    Q_D(KJob);
+
+    auto &[processed, total] = d->m_jobAmounts[unit];
+
+    const bool should_emit = (total != amount);
+
+    total = amount;
 
     if (should_emit) {
 #if KCOREADDONS_BUILD_DEPRECATED_SINCE(5, 80)
@@ -278,7 +308,7 @@ void KJob::setTotalAmount(Unit unit, qulonglong amount)
         Q_EMIT totalAmountChanged(this, unit, amount, QPrivateSignal{});
         if (unit == d->progressUnit) {
             Q_EMIT totalSize(this, amount);
-            emitPercent(d->processedAmount[unit], d->totalAmount[unit]);
+            emitPercent(processed, total);
         }
     }
 }
@@ -329,14 +359,16 @@ void KJob::emitSpeed(unsigned long value)
     Q_D(KJob);
     if (!d->speedTimer) {
         d->speedTimer = new QTimer(this);
-        connect(d->speedTimer, SIGNAL(timeout()), SLOT(_k_speedTimeout()));
+        connect(d->speedTimer, &QTimer::timeout, this, [d]() {
+            d->speedTimeout();
+        });
     }
 
     Q_EMIT speed(this, value);
     d->speedTimer->start(5000); // 5 seconds interval should be enough
 }
 
-void KJobPrivate::_k_speedTimeout()
+void KJobPrivate::speedTimeout()
 {
     Q_Q(KJob);
     // send 0 and stop the timer
