@@ -3,6 +3,7 @@
 
     SPDX-FileCopyrightText: 2007 Matthias Kretz <kretz@kde.org>
     SPDX-FileCopyrightText: 2007 Bernhard Loos <nhuh.put@web.de>
+    SPDX-FileCopyrightText: 2021 Alexander Lohnau <alexander.lohnau@gmx.de>
 
     SPDX-License-Identifier: LGPL-2.0-or-later
 */
@@ -29,6 +30,39 @@ KPluginFactory::KPluginFactory(KPluginFactoryPrivate &d)
 
 KPluginFactory::~KPluginFactory() = default;
 
+KPluginFactory::Result<KPluginFactory> KPluginFactory::loadFactory(const KPluginMetaData &data)
+{
+    Result<KPluginFactory> result;
+    QObject *obj = nullptr;
+    if (data.isStaticPlugin()) {
+        obj = data.staticPlugin().instance();
+    } else {
+        QPluginLoader loader(data.fileName());
+        obj = loader.instance();
+        if (!obj) {
+           result.errorString = tr("Could not load plugin from %1: %2").arg(data.fileName(), loader.errorString());
+           result.errorText = QStringLiteral("Could not load plugin from %1: %2").arg(data.fileName(), loader.errorString());
+           result.errorReason = INVALID_PLUGIN;
+           qCWarning(KCOREADDONS_DEBUG) << result.errorText;
+           return result;
+        }
+    }
+
+    KPluginFactory *factory = qobject_cast<KPluginFactory *>(obj);
+
+    if (factory == nullptr) {
+        result.errorString = tr("The library %1 does not offer a KPluginFactory.").arg(data.fileName());
+        result.errorReason = INVALID_FACTORY;
+        qCWarning(KCOREADDONS_DEBUG) << "Expected a KPluginFactory, got a" << obj->metaObject()->className();
+        delete obj;
+        return result;
+    }
+
+    factory->setMetaData(data);
+    result.plugin = factory;
+    return result;
+}
+
 KPluginMetaData KPluginFactory::metaData() const
 {
     Q_D(const KPluginFactory);
@@ -57,23 +91,28 @@ void KPluginFactory::registerPlugin(const QString &keyword, const QMetaObject *m
     } else {
         const QList<KPluginFactoryPrivate::Plugin> clashes(d->createInstanceHash.values(keyword));
         const QMetaObject *superClass = metaObject->superClass();
+
+        // check hierarchy of all registered with the same keyword registered classes
         if (superClass) {
             for (const KPluginFactoryPrivate::Plugin &plugin : clashes) {
                 for (const QMetaObject *otherSuper = plugin.first->superClass(); otherSuper; otherSuper = otherSuper->superClass()) {
                     if (superClass == otherSuper) {
-                        qCWarning(KCOREADDONS_DEBUG) << "Two plugins with the same interface(" << superClass->className()
-                                                     << ") were registered. Use keywords to identify the plugins.";
+                        qCWarning(KCOREADDONS_DEBUG).nospace() << "Two plugins with the same interface (" << superClass->className()
+                                                               << ") were registered in the KPluginFactory " << this->metaObject()->className() << ". "
+                                                               << "This might be due to a missing Q_OBJECT macro in one of the registered classes";
                     }
                 }
             }
         }
+        // check hierarchy of newly newly registered plugin against all registered classes with the same keyword
         for (const KPluginFactoryPrivate::Plugin &plugin : clashes) {
             superClass = plugin.first->superClass();
             if (superClass) {
                 for (const QMetaObject *otherSuper = metaObject->superClass(); otherSuper; otherSuper = otherSuper->superClass()) {
                     if (superClass == otherSuper) {
-                        qCWarning(KCOREADDONS_DEBUG) << "Two plugins with the same interface(" << superClass->className()
-                                                     << ") were registered. Use keywords to identify the plugins.";
+                        qCWarning(KCOREADDONS_DEBUG).nospace() << "Two plugins with the same interface (" << superClass->className()
+                                                               << ") were registered in the KPluginFactory " << this->metaObject()->className() << ". "
+                                                               << "This might be due to a missing Q_OBJECT macro in one of the registered classes";
                     }
                 }
             }
@@ -97,23 +136,27 @@ void KPluginFactory::registerPlugin(const QString &keyword, const QMetaObject *m
     } else {
         const QList<KPluginFactoryPrivate::PluginWithMetadata> clashes(d->createInstanceWithMetaDataHash.values(keyword));
         const QMetaObject *superClass = metaObject->superClass();
+        // check hierarchy of all registered with the same keyword registered classes
         if (superClass) {
             for (const KPluginFactoryPrivate::PluginWithMetadata &plugin : clashes) {
                 for (const QMetaObject *otherSuper = plugin.first->superClass(); otherSuper; otherSuper = otherSuper->superClass()) {
                     if (superClass == otherSuper) {
-                        qCWarning(KCOREADDONS_DEBUG) << "Two plugins with the same interface(" << superClass->className()
-                                                     << ") were registered. Use keywords to identify the plugins.";
+                        qCWarning(KCOREADDONS_DEBUG).nospace() << "Two plugins with the same interface (" << superClass->className()
+                                                               << ") were registered in the KPluginFactory " << this->metaObject()->className() << ". "
+                                                               << "This might be due to a missing Q_OBJECT macro in one of the registered classes";
                     }
                 }
             }
         }
+        // check hierarchy of newly newly registered plugin against all registered classes with the same keyword
         for (const KPluginFactoryPrivate::PluginWithMetadata &plugin : clashes) {
             superClass = plugin.first->superClass();
             if (superClass) {
                 for (const QMetaObject *otherSuper = metaObject->superClass(); otherSuper; otherSuper = otherSuper->superClass()) {
                     if (superClass == otherSuper) {
-                        qCWarning(KCOREADDONS_DEBUG) << "Two plugins with the same interface(" << superClass->className()
-                                                     << ") were registered. Use keywords to identify the plugins.";
+                        qCWarning(KCOREADDONS_DEBUG).nospace() << "Two plugins with the same interface (" << superClass->className()
+                                                               << ") were registered in the KPluginFactory " << this->metaObject()->className() << ". "
+                                                               << "This might be due to a missing Q_OBJECT macro in one of the registered classes";
                     }
                 }
             }
@@ -122,6 +165,11 @@ void KPluginFactory::registerPlugin(const QString &keyword, const QMetaObject *m
     }
 }
 
+void KPluginFactory::logFailedInstantiationMessage(KPluginMetaData data)
+{
+    qCWarning(KCOREADDONS_DEBUG) << "KPluginFactory could not load the plugin" << data.fileName();
+}
+
 #if KCOREADDONS_BUILD_DEPRECATED_SINCE(4, 0)
 QObject *KPluginFactory::createObject(QObject *parent, const char *className, const QStringList &args)
 {
